"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.diagnose_molang_math_using = exports.diagnose_molang_query_using = exports.diagnose_molang = exports.diagnose_molang_implementation = void 0;
const bc_minecraft_molang_1 = require("bc-minecraft-molang");
const Severity_1 = require("../../Types/DiagnosticsBuilder/Severity");
/**Diagnoses the given molang sets, the using party checks upon the definer if they have setup properly
 * @param using The set of molang data that is being used
 * @param definer The set of molang data that is defined
 * @param diagnoser The diagnoser to report to*/
function diagnose_molang_implementation(user, owner, ownerType, diagnoser) {
    const userid = user.id;
    const using = user.molang;
    const definer = owner.molang;
    const definerid = owner.id;
    //Is full set?
    if (bc_minecraft_molang_1.Molang.MolangFullSet.isEither(using)) {
        //Upgrade if nesscary and check
        const temp = bc_minecraft_molang_1.Molang.MolangFullSet.upgrade(definer);
        diagnose_molang_using(userid, using.geometries, definerid, temp.geometries, diagnoser, "geometry");
        diagnose_molang_using(userid, using.materials, definerid, temp.materials, diagnoser, "material");
        diagnose_molang_using(userid, using.textures, definerid, temp.textures, diagnoser, "texture");
    }
    //Check variable vs variables and such
    diagnose_molang_variable_using(userid, using.variables, definerid, definer.variables, diagnoser, ownerType);
    diagnose_molang_temp_using(using.temps, definer.temps, diagnoser, ownerType);
}
exports.diagnose_molang_implementation = diagnose_molang_implementation;
/**Diagnoses the given molang sets, the using party checks upon the definer if they have setup properly
 * @param using The set of molang data that is being used
 * @param definer The set of molang data that is defined
 * @param diagnoser The diagnoser to report to*/
function diagnose_molang(using, owner, diagnoser) {
    diagnose_molang_query_using(using, diagnoser);
    diagnose_molang_math_using(using, diagnoser);
    diagnose_molang_context_using(using, diagnoser, owner);
    diagnose_molang_allowed(using, owner, diagnoser);
}
exports.diagnose_molang = diagnose_molang;
/**Diagnoses the given using set to the given defining set
 * @param using The set of molang data that is being used
 * @param definer The set of molang data that is defined
 * @param diagnoser The diagnoser to report to
 * @param name The name of the data set such as `variable` or `query`*/
function diagnose_molang_using(userid, using, definerid, definer, diagnoser, name) {
    const checks = using.using;
    const defined1 = definer.defined;
    const defined2 = using.defined;
    for (var I = 0; I < checks.length; I++) {
        const check = checks[I];
        if (defined1.includes(check) || defined2.includes(check)) {
            //Valid
            continue;
        }
        else {
            diagnoser.Add(userid, `Missing molang defintion: ${name}.${check}\n\tUsed by: ${userid}\n\tShould be defined by ${definerid}`, Severity_1.DiagnosticSeverity.error, `molang.${name}.missing`);
        }
    }
}
function diagnose_molang_allowed(using, owner, diagnoser) {
    if (!(owner === "animation" || owner === "animation_controller"))
        return;
    const set = bc_minecraft_molang_1.Molang.MolangFullSet.harvest(using);
    if (has_any(set.textures))
        diagnoser.Add("textures.", "Animation / Animation controllers do not have acces to textures", Severity_1.DiagnosticSeverity.warning, "molang.textures.invalid");
    if (has_any(set.materials))
        diagnoser.Add("material.", "Animation / Animation controllers do not have acces to materials", Severity_1.DiagnosticSeverity.warning, "molang.material.invalid");
    if (has_any(set.geometries))
        diagnoser.Add("geometry.", "Animation / Animation controllers do not have acces to geometries", Severity_1.DiagnosticSeverity.warning, "molang.geometry.invalid");
}
function has_any(data) {
    if (bc_minecraft_molang_1.Using.is(data) && data.using.length > 0)
        return true;
    if (bc_minecraft_molang_1.Defined.is(data) && data.defined.length > 0)
        return true;
    return false;
}
function diagnose_molang_variable_using(userid, using, definerid, definer, diagnoser, owner) {
    var _a, _b;
    const checks = using.using;
    const defined1 = definer.defined;
    const defined2 = using.defined;
    for (var I = 0; I < checks.length; I++) {
        const check = checks[I];
        //Check if the variable is defined
        if (defined1.includes(check) || defined2.includes(check))
            continue;
        //Vanilla provides?
        if (InternalIdentifiable.has((_b = (_a = bc_minecraft_molang_1.MolangData.get(owner)) === null || _a === void 0 ? void 0 : _a.Variables) !== null && _b !== void 0 ? _b : [], check))
            continue;
        diagnoser.Add(userid, `Missing variable molang defintion: variable.${check}\n\tUsed by: ${userid}\n\tShould be defined by ${definerid}`, Severity_1.DiagnosticSeverity.error, `molang.variable.missing`);
    }
}
function diagnose_molang_temp_using(using, definer, diagnoser, owner) {
    var _a, _b;
    const checks = using.using;
    const defined1 = definer.defined;
    const defined2 = using.defined;
    for (var I = 0; I < checks.length; I++) {
        const check = checks[I];
        //Check if the temp is defined
        if (defined1.includes(check) || defined2.includes(check))
            continue;
        //Vanilla provides?
        if (InternalIdentifiable.has((_b = (_a = bc_minecraft_molang_1.MolangData.get(owner)) === null || _a === void 0 ? void 0 : _a.Temps) !== null && _b !== void 0 ? _b : [], check))
            continue;
        diagnoser.Add(check, `Missing molang temps defintion: ${check}`, Severity_1.DiagnosticSeverity.error, `molang.temp.missing`);
    }
}
function diagnose_molang_context_using(using, diagnoser, owner) {
    var _a, _b;
    if (typeof using === "string") {
        const out = bc_minecraft_molang_1.Using.create();
        bc_minecraft_molang_1.Molang.Types.Context.getUsing(using, out.using);
        using = out;
    }
    const checks = using.using;
    for (var I = 0; I < checks.length; I++) {
        const check = checks[I];
        //Vanilla provides?
        if (InternalIdentifiable.has((_b = (_a = bc_minecraft_molang_1.MolangData.get(owner)) === null || _a === void 0 ? void 0 : _a.Contexts) !== null && _b !== void 0 ? _b : [], check))
            continue;
        diagnoser.Add(check, `Missing molang context defintion: ${check}`, Severity_1.DiagnosticSeverity.error, `molang.context.${owner}.unknown`);
    }
}
/**
 *
 * @param using
 * @param diagnoser
 * @param owner
 */
function diagnose_molang_query_using(using, diagnoser) {
    if (typeof using === "string") {
        const out = bc_minecraft_molang_1.Using.create();
        bc_minecraft_molang_1.Molang.Types.Queries.getUsing(using, out.using);
        using = out;
    }
    const checks = using.using;
    for (let I = 0; I < checks.length; I++) {
        const check = checks[I];
        //Vanilla provides?
        if (InternalIdentifiable.has(bc_minecraft_molang_1.MolangData.General.Queries, check))
            continue;
        diagnoser.Add(check, `Unknown molang query function: ${check}`, Severity_1.DiagnosticSeverity.error, `molang.query.unknown`);
    }
}
exports.diagnose_molang_query_using = diagnose_molang_query_using;
/**
 *
 * @param using
 * @param diagnoser
 * @param owner
 */
function diagnose_molang_math_using(using, diagnoser) {
    if (typeof using === "string") {
        const out = bc_minecraft_molang_1.Using.create();
        bc_minecraft_molang_1.Molang.Types.Math.getUsing(using, out.using);
        using = out;
    }
    const checks = using.using;
    for (let I = 0; I < checks.length; I++) {
        //Too many use the CamelCase version of math >.>
        const check = checks[I].toLowerCase();
        //Vanilla provides?
        if (InternalIdentifiable.has(bc_minecraft_molang_1.MolangData.General.Math, check))
            continue;
        diagnoser.Add(check, `Unknown molang math function: ${check}`, Severity_1.DiagnosticSeverity.error, `molang.math.unknown`);
    }
}
exports.diagnose_molang_math_using = diagnose_molang_math_using;
var InternalIdentifiable;
(function (InternalIdentifiable) {
    function has(data, id) {
        for (let I = 0; I < data.length; I++) {
            if (data[I].id.startsWith(id))
                return true;
        }
        return false;
    }
    InternalIdentifiable.has = has;
})(InternalIdentifiable || (InternalIdentifiable = {}));
//# sourceMappingURL=diagnostics.js.map