"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnoser = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const path = require("path");
const BehaviorPack_1 = require("../../Diagnostics/BehaviorPack/BehaviorPack");
const SkinPack_1 = require("../../Diagnostics/SkinPack/SkinPack");
const WorldPack_1 = require("../../Diagnostics/WorldPack/WorldPack");
const diagnose_1 = require("../../Diagnostics/Format/diagnose");
const Language_1 = require("../../Diagnostics/Minecraft/Language");
const ResourcePack_1 = require("../../Diagnostics/ResourcePack/ResourcePack");
const Severity_1 = require("../DiagnosticsBuilder/Severity");
/**The object that is responsible for diagnosing minecraft bedrock files*/
class Diagnoser {
    /**Create a new instance of Diagnoser
     * @param context The context needed to perform diagnostics*/
    constructor(context) {
        this.context = context;
    }
    /** Process and diagnoses the given document
     * @param doc The textdocument to process or the uri to the document
     * @returns `true` or `false` if the diagnostics was successfull*/
    Process(doc) {
        if (typeof doc === "string") {
            const temp = this.context.getDocument(doc);
            if (!temp)
                return false;
            doc = temp;
        }
        const pack = this.context.getCache().get(doc);
        if (!pack)
            return false;
        //Check if diagnostics was disabled
        const ext = path.extname(doc.uri);
        //Check if diagnostics for this file type is disabled
        if (pack.context.attributes["diagnostic" + ext] === "false")
            return false;
        const diagnoser = this.context.getDiagnoser(doc, pack.context);
        if (!diagnoser)
            return false;
        let out = false;
        try {
            //diagnose path
            (0, diagnose_1.format_diagnose_path)(pack, doc.uri, diagnoser);
            //Language file?
            if (doc.uri.endsWith(".lang")) {
                (0, Language_1.minecraft_language_diagnose)(doc, diagnoser);
            }
            else {
                //Check per pack
                switch (bc_minecraft_bedrock_project_1.PackType.detect(doc.uri)) {
                    case bc_minecraft_bedrock_project_1.PackType.behavior_pack:
                        out = BehaviorPack_1.BehaviorPack.Process(doc, diagnoser);
                        break;
                    case bc_minecraft_bedrock_project_1.PackType.resource_pack:
                        out = ResourcePack_1.ResourcePack.Process(doc, diagnoser);
                        break;
                    case bc_minecraft_bedrock_project_1.PackType.skin_pack:
                        out = SkinPack_1.SkinPack.Process(doc, diagnoser);
                        break;
                    case bc_minecraft_bedrock_project_1.PackType.world:
                        out = WorldPack_1.WorldPack.Process(doc, diagnoser);
                        break;
                }
            }
        }
        catch (err) {
            let msg;
            if (err.message && err.stack) {
                msg = `${err.message}\n\t${err.stack}`;
            }
            else {
                msg = JSON.stringify(err);
            }
            diagnoser.Add({ character: 0, line: 0 }, msg, Severity_1.DiagnosticSeverity.error, "diagnoser.internal.exception");
        }
        diagnoser.done();
        return out;
    }
    /**Diagnoses the entire given folder
     * @param folder The folder to retrieve files of
     * @param ignores The pattern to ignore on files*/
    ProcessFolder(folder, ignores) {
        const files = this.context.getFiles(folder, ["*"], ignores);
        let out = false;
        for (let I = 0; I < files.length; I++) {
            out || (out = this.Process(files[I]));
        }
        return out;
    }
    /**Process the entire given pack
     * @param pack The pack to process
     * @returns True or false is something was processed*/
    ProcessPack(pack) {
        return this.ProcessFolder(pack.folder, pack.context.ignores);
    }
}
exports.Diagnoser = Diagnoser;
//# sourceMappingURL=Diagnoser.js.map